import fred
import requests
import json
import pandas

# local app
import settings

class Rates(object):
    def __init__(self, OXE_API=False, FRED_API=False):
        self.df = self.ratesdf()
        if OXE_API:
            settings.OPENEXCHANGE_APP_ID = OXE_API
        if FRED_API:
            settings.FRED_KEY = FRED_API
        self.rebase()
        
    def get_hour_value(self):
        fred.key(settings.FRED_KEY)
        last_observation = fred.observations(
			settings.FRED_SERIES)['observations'][-1]
        h = last_observation['value']
        try:
            return float(h)
        except:
            return settings.DEFAULT_HOUR_VALUE_IN_USD

    def get_currency_rates(self):
        r = requests.get("%s?app_id=%s&base=%s" % (settings.OPENEXCHANGE_URL,
                                                   settings.OPENEXCHANGE_APP_ID,
                                                   settings.OPENEXCHANGE_BASE_CURRENCY))
        return json.loads(r.content)

    def ratesdf(self,base='h'):
        self.currency_rates = self.get_currency_rates()
        df = pandas.DataFrame.from_records([self.currency_rates['rates']])
        df.columns = [col.lower() for col in df.columns]
        return df
    
    def rebase(self,base='h'):
        self.hour_value = self.get_hour_value()
        # We express everything interms of hours
        self.df = self.df*self.hour_value
        self.df['h'] = 1.



# These are only needed, if you are using custom conversion rates.
# If you are valuating everything in terms of work hour value,
# then you don't need to enter this information.

# Currency value ( get one at openexchangerates.org )
OPENEXCHANGE_APP_ID = ''
OPENEXCHANGE_BASE_CURRENCY = 'USD'
OPENEXCHANGE_URL = 'https://openexchangerates.org/api/latest.json'

# [Optional] Hour value ( get one at api.stlouisfed.org )
FRED_KEY = ''
FRED_SERIES = 'CES0500000003'
DEFAULT_HOUR_VALUE_IN_USD = 25.0




import pandas
import utils

class PlanMaker(object):

    def __init__(self, 
                 plan_tuples,
                 conversion_rates=pandas.DataFrame({'h': [1.]})):

        """
        Example of plan_tuples:

        >>> breakfast = [
             ('time: 0.003~0.004@24h; loaf of black bread: 1@0.1h; butter grams: 15@0.01h; tomato: 0.5@0.2h',
              'sandwitch: 1@0.12h'),
             ('eggs: 2~5@0.012h; scrambling actions: 50~100; time: 0.003~0.005@24h',
              'scrambled egg servings: 1@0.16h'),
             ('coffee teaspoon: 1~2@0.00012h; liters of water: 0.2~0.3@0.0001h; time: 0.003~0.005@24h',
              'cup of coffee: 1~1.5@0.08h')
        ]
        
        Note: if richer currency conversion_rates is provided, then it can be something like:

        >>> breakfast_with_currencies = [
            ('time: 0.003~0.004@24h; loaf of black bread: 1@2.22eur; butter grams: 15@0.25usd; tomato: 0.5@5usd',
             'sandwitch: 1@3usd'),
            ('eggs: 2~5@0.3usd; scrambling actions: 50~100; time: 0.003~0.005@24h',
             'scrambled egg servings: 1@4usd'),
            ('coffee teaspoon: 1~2@0.003usd; liters of water: 0.2~0.3@0.0025usd; time: 0.003~0.005@24h',
             'cup of coffee: 1~1.5@2usd')
        ]

        """
        self.df = utils.plandf(plan_tuples)
        self.df.columns.names = [None, 'Step #ID']
        if 'h' not in conversion_rates.columns:
            raise Exception("Column 'h' (hour value) must be among columns of conversion_rates. 'h' not found.")
        else:
            try:
                self.rates = conversion_rates/conversion_rates['h'].values[0]
            except:
                raise Exception("Could not normalize by hour value. Check if conversion rates are numeric.")

    def get_price_unit_values(self, default=pandas.np.nan):

        price_units = utils.subdf(self.df, 'price_unit', 'price_unit')    

        price_unit_values = price_units.applymap(
            lambda x: self.rates[x].values[0] if x in self.rates.columns
            else pandas.np.nan).apply(pandas.to_numeric,
                                      errors='coerce')

        return price_unit_values.fillna(default)

    def get_asset_values(self, case='MEAN'):

        if case=='WORST':

            units = utils.subdf(self.df,
                          input_key='max_units',
                          output_key='min_units',
                          to_numeric=True)

            prices = utils.subdf(self.df,
                           input_key='max_price',
                           output_key='min_price',
                           to_numeric=True)

        elif case=='BEST':

            units = utils.subdf(self.df,
                          input_key='min_units',
                          output_key='max_units',
                          to_numeric=True)

            prices = utils.subdf(self.df,
                           input_key='min_price',
                           output_key='max_price',
                           to_numeric=True)

        else: # ('MEAN')

            units = ( utils.subdf(self.df,
                          input_key='min_units',
                          output_key='min_units',
                          to_numeric=True) +
                      utils.subdf(self.df,
                          input_key='max_units',
                          output_key='max_units',
                          to_numeric=True) ) / 2.
			# Add known means, if 'f_units' is a number:
            known_units_means = utils.subdf(self.df, input_key='f_units',
							  output_key='f_units',to_numeric=True)
            # add dataframe with 'f_units', and fillna(.0)
            units = utils.replace_known_numeric_values(units, known_units_means)

            prices = ( utils.subdf(self.df,
                           input_key='min_price',
                           output_key='min_price',
                           to_numeric=True) +
                       utils.subdf(self.df,
                           input_key='max_price',
                           output_key='max_price',
                           to_numeric=True) ) / 2.
			# Add known means, if 'f_price' is a number:
            known_price_means = utils.subdf(self.df, input_key='f_price',
							  output_key='f_price',to_numeric=True)
            # add dataframe with 'f_units', and fillna(.0)
            prices = utils.replace_known_numeric_values(prices, known_price_means)

        return units * prices * self.get_price_unit_values()

    
    def case_value(self, case='WORST'):

        io_df = self.get_asset_values(case=case)
        
        values = utils.set_axis(io_df).fillna(0.)

        value = pandas.concat({'input': values['input'].fillna(0.).sum(axis=1),
                               'output': values['output'].fillna(0.).sum(axis=1)},
                                axis=1)



        v = value.cumsum()

        return (v['output']-v['input'])
    
    
    def get_scenarios(self):
        return pandas.concat([self.case_value('WORST'),
                             self.case_value('MEAN'),
                             self.case_value('BEST')],
                             axis=1,
                             keys=['worst', 'mean','best']
        )



from plan_maker import PlanMaker

def read(plan_tuples, conversion_rates=False, scenarios=True):

    if not isinstance(conversion_rates, bool):
        try:
            p = PlanMaker(plan_tuples, conversion_rates)
        except:
            print "Could not retrieve currency conversion rates." 
            p = False

    else:
		p = PlanMaker(plan_tuples)

    if p:
        if scenarios:
            return p.get_scenarios()
        else:
            return p
    else:
		print "Could not read the plan_tuples."
		return False

import fred
import requests
import pandas as pd

class Plan(object):
    def __init__(self):
        self.set_hour_rate()
        self.set_currenc_rates()

    def set_hour_rate(self, h=None):
        if h:
            self.hour = h
            print "Hour value of work (self.hour) was set to %s usd from FRED API. Retrieving currency rates.." % self.hour
            self.set_currenc_rates()
            print "Done."
        else:
            fred.key('0c9a5ec8dd8c63ab8cbec6514a8f5b37')
            last_observation = fred.observations(
                settings.FRED_SERIES)['observations'][-1]
            h = last_observation['value']
            try:
                self.hour = float(h)
                print "Hour value of work (self.hour) was set to %s usd from FRED API." % self.hour
            except:
                self.hour = 25.
                print "Failed to retrieve rates from FRED API. Assuming 1h = 25usd."

    def set_currenc_rates(self):
        currency_rates = requests.get('http://api.fixer.io/latest?base=USD').json()['rates']
        self.rates = pd.DataFrame( dict({'h': [self.hour], 'usd': 1.}, **{key.lower(): 1/currency_rates[key] for ix, key in enumerate(currency_rates)} ) )
        print "Currency values had been set from FIXER IO, check the .rates attribute.\nThe currency 'h' means the time of 1 hour labor, based on FRED API."

    def from_records(self, plan_dict):
        self.info = read([(step['input'], step['output']) for step in plan_dict], 
                       conversion_rates=self.rates,
                       scenarios=False)# * (rates['h'] / rates['gbp']).values[0]
        self.df = self.info.get_scenarios()
        return self.df

    def steps(self, plan_dict):
        return self.from_records(plan_dict)

    def plot(self, figsize=(10,4.5)):
        self.df['worst'].dropna().plot(marker='.', figsize=figsize)
        self.df['mean'].dropna().plot(marker='.', figsize=figsize)
        p = self.df['best'].dropna().plot(marker='.', figsize=figsize)
        p.set_ylabel('time (h, as value)')
        p.set_xlabel('time (h, elapsed)');
        p.grid(True)
